/**
 * Package that contains interfaces that define how to implement
 * functionality for dynamically resolving type during deserialization.
 * This is needed for complete handling of polymorphic types, where
 * actual type can not be determined statically (declared type is
 * a supertype of actual polymorphic serialized types).
 *
 * @since 1.5
 */
package org.codehaus.jackson.map.jsontype;
