/**
 * Package that implements "interface materializer" functionality, whereby
 * abstract classes and interfaces can be used as-is, and framework constructs
 * implementations as needed.
 */
package org.codehaus.jackson.mrbean;
